@@ -111,7 +111,8 @@ void BtSetup::setup(std::vector<Command*>& commands,
   }
   {
     ActivePeerConnectionCommand* c =
-      new ActivePeerConnectionCommand(e->newCUID(), requestGroup, e, 10);
+      new ActivePeerConnectionCommand(e->newCUID(), requestGroup, e,
+                                      metadataGetMode?2:10);
     c->setBtRuntime(btRuntime);
     c->setPieceStorage(pieceStorage);
     c->setPeerStorage(peerStorage);
@@ -149,7 +150,7 @@ void BtSetup::setup(std::vector<Command*>& commands,
         unionCri->addSeedCriteria(cri);
       }
     }
-    if(unionCri->getSeedCriterion().size() > 0) {
+    if(!unionCri->getSeedCriterion().empty()) {
       SeedCheckCommand* c =
         new SeedCheckCommand(e->newCUID(), requestGroup, e, unionCri);
       c->setPieceStorage(pieceStorage);
