@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -36,70 +36,49 @@
 #define _D_FEATURE_CONFIG_H_
 
 #include "common.h"
-#include <map>
 
-typedef map<string, int32_t> PortMap;
-typedef map<string, bool> FeatureMap;
+#include <map>
+#include <string>
+
+#include "SharedHandle.h"
+
+namespace aria2 {
+
+typedef std::map<std::string, uint16_t> PortMap;
+typedef std::map<std::string, bool> FeatureMap;
 
 class FeatureConfig {
 private:
-  static FeatureConfig* featureConfig;
+  static SharedHandle<FeatureConfig> _featureConfig;
 
-  PortMap defaultPorts;
-  FeatureMap supportedFeatures;
-  Strings features;
+  PortMap _defaultPorts;
+  FeatureMap _features;
 
   FeatureConfig();
-  ~FeatureConfig() {}
 public:
-  static FeatureConfig* getInstance() {
-    if(!featureConfig) {
-      featureConfig = new FeatureConfig();
-    }
-    return featureConfig;
+  static const SharedHandle<FeatureConfig>& getInstance();
+
+  uint16_t getDefaultPort(const std::string& protocol) const;
+
+  bool isSupported(const std::string& feature) const;
+
+  std::string featureSummary() const;
+
+  const FeatureMap& getFeatures() const
+  {
+    return _features;
   }
 
-  static void release() {
-    delete featureConfig;
-    featureConfig = 0;
-  }
-
-  int32_t getDefaultPort(const string& protocol) const {
-    PortMap::const_iterator itr = defaultPorts.find(protocol);
-    if(itr == defaultPorts.end()) {
-      return 0;
-    } else {
-      return itr->second;
-    }
-  }
-
-  bool isSupported(const string& feature) const {
-    FeatureMap::const_iterator itr = supportedFeatures.find(feature);
-    if(itr == supportedFeatures.end()) {
-      return false;
-    } else {
-      return itr->second;
-    }
-  }
-
-  const Strings& getFeatures() const {
-    return features;
-  }
-
-  string getConfigurationSummary() const {
-    string summary;
-    for(Strings::const_iterator itr = features.begin();
-	itr != features.end(); itr++) {
-      summary += *itr;
-      if(isSupported(*itr)) {
-	summary += ": yes";
-      } else {
-	summary += ": no";
-      }
-      summary += "\n";
-    }
-    return summary;
-  }
+  static const std::string FEATURE_HTTPS;
+  static const std::string FEATURE_BITTORRENT;
+  static const std::string FEATURE_METALINK;
+  static const std::string FEATURE_MESSAGE_DIGEST;
+  static const std::string FEATURE_ASYNC_DNS;
+  static const std::string FEATURE_XML_RPC;
+  static const std::string FEATURE_GZIP;
+  static const std::string FEATURE_FIREFOX3_COOKIE;
 };
 
+} // namespace aria2
+
 #endif // _D_FEATURE_CONFIG_H_
