@@ -62,8 +62,8 @@ const std::string FtpConnection::A("A");
 
 const std::string FtpConnection::I("I");
 
-FtpConnection::FtpConnection(int32_t cuid, const SocketHandle& socket,
-                             const RequestHandle& req,
+FtpConnection::FtpConnection(cuid_t cuid, const SocketHandle& socket,
+                             const SharedHandle<Request>& req,
                              const SharedHandle<AuthConfig>& authConfig,
                              const Option* op):
   cuid(cuid), socket(socket), req(req),
@@ -80,8 +80,11 @@ bool FtpConnection::sendUser()
     std::string request = "USER ";
     request += _authConfig->getUser();
     request += "\r\n";
-    logger->info(MSG_SENDING_REQUEST, cuid, "USER ********");
-    _socketBuffer.feedSendBuffer(request);
+    if(logger->info()) {
+      logger->info(MSG_SENDING_REQUEST,
+                   util::itos(cuid).c_str(),"USER ********");
+    }
+    _socketBuffer.pushStr(request);
   }
   _socketBuffer.send();
   return _socketBuffer.sendBufferIsEmpty();
@@ -93,8 +96,11 @@ bool FtpConnection::sendPass()
     std::string request = "PASS ";
     request += _authConfig->getPassword();
     request += "\r\n";
-    logger->info(MSG_SENDING_REQUEST, cuid, "PASS ********");
-    _socketBuffer.feedSendBuffer(request);
+    if(logger->info()) {
+      logger->info(MSG_SENDING_REQUEST,
+                   util::itos(cuid).c_str(),"PASS ********");
+    }
+    _socketBuffer.pushStr(request);
   }
   _socketBuffer.send();
   return _socketBuffer.sendBufferIsEmpty();
@@ -112,8 +118,11 @@ bool FtpConnection::sendType()
     std::string request = "TYPE ";
     request += type;
     request += "\r\n";
-    logger->info(MSG_SENDING_REQUEST, cuid, request.c_str());
-    _socketBuffer.feedSendBuffer(request);
+    if(logger->info()) {
+      logger->info(MSG_SENDING_REQUEST,
+                   util::itos(cuid).c_str(),request.c_str());
+    }
+    _socketBuffer.pushStr(request);
   }
   _socketBuffer.send();
   return _socketBuffer.sendBufferIsEmpty();
@@ -123,8 +132,11 @@ bool FtpConnection::sendPwd()
 {
   if(_socketBuffer.sendBufferIsEmpty()) {
     std::string request = "PWD\r\n";
-    logger->info(MSG_SENDING_REQUEST, cuid, request.c_str());
-    _socketBuffer.feedSendBuffer(request);
+    if(logger->info()) {
+      logger->info(MSG_SENDING_REQUEST,
+                   util::itos(cuid).c_str(),request.c_str());
+    }
+    _socketBuffer.pushStr(request);
   }
   _socketBuffer.send();
   return _socketBuffer.sendBufferIsEmpty();
@@ -133,16 +145,21 @@ bool FtpConnection::sendPwd()
 bool FtpConnection::sendCwd()
 {
   if(_socketBuffer.sendBufferIsEmpty()) {
-    logger->info("CUID#%d - Using base working directory '%s'",
-                 cuid, _baseWorkingDir.c_str());
+    if(logger->info()) {
+      logger->info("CUID#%s - Using base working directory '%s'",
+                   util::itos(cuid).c_str(), _baseWorkingDir.c_str());
+    }
     std::string request = "CWD ";
     if(_baseWorkingDir != "/") {
       request += _baseWorkingDir;
     }
-    request += util::urldecode(req->getDir());
+    request += util::percentDecode(req->getDir());
     request += "\r\n";
-    logger->info(MSG_SENDING_REQUEST, cuid, request.c_str());
-    _socketBuffer.feedSendBuffer(request);
+    if(logger->info()) {
+      logger->info(MSG_SENDING_REQUEST,
+                   util::itos(cuid).c_str(),request.c_str());
+    }
+    _socketBuffer.pushStr(request);
   }
   _socketBuffer.send();
   return _socketBuffer.sendBufferIsEmpty();
@@ -152,10 +169,13 @@ bool FtpConnection::sendMdtm()
 {
   if(_socketBuffer.sendBufferIsEmpty()) {
     std::string request = "MDTM ";
-    request += util::urlencode(req->getFile());
+    request += util::percentDecode(req->getFile());
     request += "\r\n";
-    logger->info(MSG_SENDING_REQUEST, cuid, request.c_str());
-    _socketBuffer.feedSendBuffer(request);
+    if(logger->info()) {
+      logger->info(MSG_SENDING_REQUEST,
+                   util::itos(cuid).c_str(), request.c_str());
+    }
+    _socketBuffer.pushStr(request);
   }
   _socketBuffer.send();
   return _socketBuffer.sendBufferIsEmpty();
@@ -165,10 +185,13 @@ bool FtpConnection::sendSize()
 {
   if(_socketBuffer.sendBufferIsEmpty()) {
     std::string request = "SIZE ";
-    request += util::urldecode(req->getFile());
+    request += util::percentDecode(req->getFile());
     request += "\r\n";
-    logger->info(MSG_SENDING_REQUEST, cuid, request.c_str());
-    _socketBuffer.feedSendBuffer(request);
+    if(logger->info()) {
+      logger->info(MSG_SENDING_REQUEST,
+                   util::itos(cuid).c_str(), request.c_str());
+    }
+    _socketBuffer.pushStr(request);
   }
   _socketBuffer.send();
   return _socketBuffer.sendBufferIsEmpty();
@@ -178,8 +201,11 @@ bool FtpConnection::sendPasv()
 {
   if(_socketBuffer.sendBufferIsEmpty()) {
     static const std::string request("PASV\r\n");
-    logger->info(MSG_SENDING_REQUEST, cuid, request.c_str());
-    _socketBuffer.feedSendBuffer(request);
+    if(logger->info()) {
+      logger->info(MSG_SENDING_REQUEST,
+                   util::itos(cuid).c_str(), request.c_str());
+    }
+    _socketBuffer.pushStr(request);
   }
   _socketBuffer.send();
   return _socketBuffer.sendBufferIsEmpty();
@@ -216,14 +242,17 @@ bool FtpConnection::sendPort(const SharedHandle<SocketCore>& serverSocket)
     request += ",";
     request += util::uitos(addrinfo.second%256);
     request += "\r\n";
-    logger->info(MSG_SENDING_REQUEST, cuid, request.c_str());
-    _socketBuffer.feedSendBuffer(request);
+    if(logger->info()) {
+      logger->info(MSG_SENDING_REQUEST,
+                   util::itos(cuid).c_str(), request.c_str());
+    }
+    _socketBuffer.pushStr(request);
   }
   _socketBuffer.send();
   return _socketBuffer.sendBufferIsEmpty();
 }
 
-bool FtpConnection::sendRest(const SegmentHandle& segment)
+bool FtpConnection::sendRest(const SharedHandle<Segment>& segment)
 {
   if(_socketBuffer.sendBufferIsEmpty()) {
     std::string request = "REST ";
@@ -232,10 +261,12 @@ bool FtpConnection::sendRest(const SegmentHandle& segment)
     } else {
       request += util::itos(segment->getPositionToWrite());
     }
-    request += "\r\n";
-    
-    logger->info(MSG_SENDING_REQUEST, cuid, request.c_str());
-    _socketBuffer.feedSendBuffer(request);
+    request += "\r\n";    
+    if(logger->info()) {
+      logger->info(MSG_SENDING_REQUEST,
+                   util::itos(cuid).c_str(), request.c_str());
+    }
+    _socketBuffer.pushStr(request);
   }
   _socketBuffer.send();
   return _socketBuffer.sendBufferIsEmpty();
@@ -245,10 +276,13 @@ bool FtpConnection::sendRetr()
 {
   if(_socketBuffer.sendBufferIsEmpty()) {
     std::string request = "RETR ";
-    request += util::urldecode(req->getFile());
+    request += util::percentDecode(req->getFile());
     request += "\r\n";
-    logger->info(MSG_SENDING_REQUEST, cuid, request.c_str());
-    _socketBuffer.feedSendBuffer(request);
+    if(logger->info()) {
+      logger->info(MSG_SENDING_REQUEST,
+                   util::itos(cuid).c_str(), request.c_str());
+    }
+    _socketBuffer.pushStr(request);
   }
   _socketBuffer.send();
   return _socketBuffer.sendBufferIsEmpty();
@@ -341,7 +375,10 @@ bool FtpConnection::bulkReceiveResponse(std::pair<unsigned int, std::string>& re
   if((length = findEndOfResponse(status, strbuf)) != std::string::npos) {
     response.first = status;
     response.second = strbuf.substr(0, length);
-    logger->info(MSG_RECEIVE_RESPONSE, cuid, response.second.c_str());
+    if(logger->info()) {
+      logger->info(MSG_RECEIVE_RESPONSE,
+                   util::itos(cuid).c_str(), response.second.c_str());
+    }
     strbuf.erase(0, length);
     return true;
   } else {
