@@ -79,7 +79,7 @@ void HttpResponse::validateResponse() const
     // compare the received range against the requested range
     RangeHandle responseRange = httpHeader->getRange();
     if(!httpRequest->isRangeSatisfied(responseRange)) {
-      throw DL_ABORT_EX
+      throw DL_ABORT_EX2
         (StringFormat
          (EX_INVALID_RANGE_HEADER,
           util::itos(httpRequest->getStartByte(), true).c_str(),
@@ -87,7 +87,8 @@ void HttpResponse::validateResponse() const
           util::uitos(httpRequest->getEntityLength(), true).c_str(),
           util::itos(responseRange->getStartByte(), true).c_str(),
           util::itos(responseRange->getEndByte(), true).c_str(),
-          util::uitos(responseRange->getEntityLength(), true).c_str()).str());
+          util::uitos(responseRange->getEntityLength(), true).c_str()).str(),
+         downloadresultcode::CANNOT_RESUME);
     }
   }
 }
@@ -105,8 +106,10 @@ std::string HttpResponse::determinFilename() const
       return file;
     }
   } else {
-    logger->info(MSG_CONTENT_DISPOSITION_DETECTED,
-                 cuid, contentDisposition.c_str());
+    if(logger->info()) {
+      logger->info(MSG_CONTENT_DISPOSITION_DETECTED,
+                   util::itos(cuid).c_str(), contentDisposition.c_str());
+    }
     return contentDisposition;
   }
 }
@@ -133,12 +136,14 @@ void HttpResponse::processRedirect()
 {
   
   if(httpRequest->getRequest()->redirectUri(getRedirectURI())) {
-    logger->info(MSG_REDIRECT, cuid,
-                 httpRequest->getRequest()->getCurrentUri().c_str());
+    if(logger->info()) {
+      logger->info(MSG_REDIRECT, util::itos(cuid).c_str(),
+                   httpRequest->getRequest()->getCurrentUri().c_str());
+    }
   } else {
     throw DL_RETRY_EX
-      (StringFormat("CUID#%d - Redirect to %s failed. It may not be a valid"
-                    " URI.", cuid,
+      (StringFormat("CUID#%s - Redirect to %s failed. It may not be a valid"
+                    " URI.", util::itos(cuid).c_str(),
                     httpRequest->getRequest()->getCurrentUri().c_str()).str());
   }
 }
