@@ -76,9 +76,9 @@
 namespace aria2 {
 
 PeerInteractionCommand::PeerInteractionCommand
-(int32_t cuid,
+(cuid_t cuid,
  RequestGroup* requestGroup,
- const PeerHandle& p,
+ const SharedHandle<Peer>& p,
  DownloadEngine* e,
  const SharedHandle<BtRuntime>& btRuntime,
  const SharedHandle<PieceStorage>& pieceStorage,
@@ -189,7 +189,8 @@ PeerInteractionCommand::PeerInteractionCommand
     (getOption()->getAsInt(PREF_BT_KEEP_ALIVE_INTERVAL));
   btInteractive->setRequestGroupMan(e->_requestGroupMan);
   btInteractive->setBtMessageFactory(factory);
-  if(metadataGetMode || torrentAttrs[bittorrent::PRIVATE].i() == 0) {
+  if((metadataGetMode || torrentAttrs[bittorrent::PRIVATE].i() == 0) &&
+     !peer->isLocalPeer()) {
     if(getOption()->getAsBool(PREF_ENABLE_PEER_EXCHANGE)) {
       btInteractive->setUTPexEnabled(true);
     }
@@ -243,8 +244,6 @@ PeerInteractionCommand::~PeerInteractionCommand() {
 
   _requestGroup->decreaseNumCommand();
   _btRuntime->decreaseConnections();
-  //logger->debug("CUID#%d - unregistered message factory using ID:%s",
-  //cuid, peer->getId().c_str());
 }
 
 bool PeerInteractionCommand::executeInternal() {
@@ -324,7 +323,7 @@ bool PeerInteractionCommand::executeInternal() {
 // TODO this method removed when PeerBalancerCommand is implemented
 bool PeerInteractionCommand::prepareForNextPeer(time_t wait) {
   if(_peerStorage->isPeerAvailable() && _btRuntime->lessThanEqMinPeers()) {
-    PeerHandle peer = _peerStorage->getUnusedPeer();
+    SharedHandle<Peer> peer = _peerStorage->getUnusedPeer();
     peer->usedBy(e->newCUID());
     PeerInitiateConnectionCommand* command =
       new PeerInitiateConnectionCommand
