@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -33,5 +33,50 @@
  */
 /* copyright --> */
 #include "SimpleRandomizer.h"
+#include "a2time.h"
+#include <cstdlib>
+#include <sys/types.h>
+#include <unistd.h>
 
-RandomizerHandle SimpleRandomizer::randomizer = 0;
+namespace aria2 {
+
+SharedHandle<SimpleRandomizer> SimpleRandomizer::_randomizer;
+
+const SharedHandle<SimpleRandomizer>& SimpleRandomizer::getInstance()
+{
+  if(_randomizer.isNull()) {
+    _randomizer.reset(new SimpleRandomizer());
+  }
+  return _randomizer;
+}
+  
+void SimpleRandomizer::init()
+{
+  srand(time(0)^getpid());
+}
+
+SimpleRandomizer::SimpleRandomizer() {}
+
+SimpleRandomizer::~SimpleRandomizer() {}
+
+long int SimpleRandomizer::getRandomNumber()
+{
+  return rand();
+}
+
+long int SimpleRandomizer::getMaxRandomNumber()
+{
+  return RAND_MAX;
+}
+
+long int SimpleRandomizer::getRandomNumber(long int to)
+{
+  return(int32_t)(((double)to)*getRandomNumber()/(getMaxRandomNumber()+1.0));
+}
+
+long int SimpleRandomizer::operator()(long int to)
+{
+  return getRandomNumber(to);
+}
+
+} // namespace aria2
