@@ -38,6 +38,7 @@
 #include <algorithm>
 
 #include "Logger.h"
+#include "LogFactory.h"
 #include "BDE.h"
 #include "DlAbortEx.h"
 #include "Option.h"
@@ -61,6 +62,8 @@
 #include "array_fun.h"
 #include "XmlRpcMethodFactory.h"
 #include "XmlRpcResponse.h"
+#include "SegmentMan.h"
+#include "TimedHaltCommand.h"
 #ifdef ENABLE_BITTORRENT
 # include "bittorrent_helper.h"
 # include "BtRegistry.h"
@@ -80,9 +83,11 @@ const BDE BDE_FALSE = BDE("false");
 const BDE BDE_OK = BDE("OK");
 const BDE BDE_ACTIVE = BDE("active");
 const BDE BDE_WAITING = BDE("waiting");
+const BDE BDE_PAUSED = BDE("paused");
 const BDE BDE_REMOVED = BDE("removed");
 const BDE BDE_ERROR = BDE("error");
 const BDE BDE_COMPLETE = BDE("complete");
+const BDE BDE_USED = BDE("used");
 
 const std::string KEY_GID = "gid";
 const std::string KEY_ERROR_CODE = "errorCode";
@@ -111,6 +116,7 @@ const std::string KEY_PATH = "path";
 const std::string KEY_SELECTED = "selected";
 const std::string KEY_LENGTH = "length";
 const std::string KEY_URI = "uri";
+const std::string KEY_CURRENT_URI = "currentUri";
 const std::string KEY_VERSION = "version";
 const std::string KEY_ENABLED_FEATURES = "enabledFeatures";
 const std::string KEY_METHOD_NAME = "methodName";
@@ -126,9 +132,10 @@ const std::string KEY_ANNOUNCE_LIST = "announceList";
 const std::string KEY_COMMENT = "comment";
 const std::string KEY_CREATION_DATE = "creationDate";
 const std::string KEY_MODE = "mode";
+const std::string KEY_SERVERS = "servers";
 }
 
-static BDE createGIDResponse(int32_t gid)
+static BDE createGIDResponse(gid_t gid)
 {
   return BDE(util::itos(gid));
 }
@@ -145,6 +152,17 @@ static BDE addRequestGroup(const SharedHandle<RequestGroup>& group,
   return createGIDResponse(group->getGID());
 }
 
+static
+SharedHandle<RequestGroup>
+findRequestGroup(const SharedHandle<RequestGroupMan>& rgman, gid_t gid)
+{
+  SharedHandle<RequestGroup> group = rgman->findRequestGroup(gid);
+  if(group.isNull()) {
+    group = rgman->findReservedGroup(gid);
+  }
+  return group;
+}
+
 static bool hasDictParam(const BDE& params, size_t index)
 {
   return params.size() > index && params[index].isDict();
@@ -165,6 +183,17 @@ static void getPosParam(const BDE& params, size_t posParamIndex,
   }
 } 
 
+template<typename OutputIterator>
+static void extractUris(OutputIterator out, const BDE& src)
+{
+  for(BDE::List::const_iterator i = src.listBegin(), eoi = src.listEnd();
+      i != eoi; ++i) {
+    if((*i).isString()) {
+      out++ = (*i).s();
+    }
+  }
+}
+
 BDE AddUriXmlRpcMethod::process(const XmlRpcRequest& req, DownloadEngine* e)
 {
   const BDE& params = req._params;
@@ -172,13 +201,8 @@ BDE AddUriXmlRpcMethod::process(const XmlRpcRequest& req, DownloadEngine* e)
   if(params.empty() || !params[0].isList() || params[0].empty()) {
     throw DL_ABORT_EX("URI is not provided.");
   }
-  std::deque<std::string> uris;
-  for(BDE::List::const_iterator i = params[0].listBegin();
-      i != params[0].listEnd(); ++i) {
-    if((*i).isString()) {
-      uris.push_back((*i).s());
-    }
-  }
+  std::vector<std::string> uris;
+  extractUris(std::back_inserter(uris), params[0]);
 
   SharedHandle<Option> requestOption(new Option(*e->option));
   if(hasDictParam(params, 1)) {
@@ -188,7 +212,7 @@ BDE AddUriXmlRpcMethod::process(const XmlRpcRequest& req, DownloadEngine* e)
   bool posGiven = false;
   getPosParam(params, 2, posGiven, pos);
 
-  std::deque<SharedHandle<RequestGroup> > result;
+  std::vector<SharedHandle<RequestGroup> > result;
   createRequestGroupForUri(result, requestOption, uris,
                            /* ignoreForceSeq = */ true,
                            /* ignoreLocalPath = */ true);
@@ -210,14 +234,9 @@ BDE AddTorrentXmlRpcMethod::process
     throw DL_ABORT_EX("Torrent data is not provided.");
   }
   
-  std::deque<std::string> uris;
+  std::vector<std::string> uris;
   if(params.size() > 1 && params[1].isList()) {
-    for(BDE::List::const_iterator i = params[1].listBegin();
-        i != params[1].listEnd(); ++i) {
-      if((*i).isString()) {
-        uris.push_back((*i).s());
-      }
-    }
+    extractUris(std::back_inserter(uris), params[1]);
   }
   SharedHandle<Option> requestOption(new Option(*e->option));
   if(hasDictParam(params, 2)) {
@@ -227,7 +246,7 @@ BDE AddTorrentXmlRpcMethod::process
   bool posGiven = false;
   getPosParam(params, 3, posGiven, pos);
 
-  std::deque<SharedHandle<RequestGroup> > result;
+  std::vector<SharedHandle<RequestGroup> > result;
   createRequestGroupForBitTorrent(result, requestOption,
                                   uris,
                                   params[0].s());
@@ -258,7 +277,7 @@ BDE AddMetalinkXmlRpcMethod::process
   bool posGiven = false;
   getPosParam(params, 2, posGiven, pos);
 
-  std::deque<SharedHandle<RequestGroup> > result;
+  std::vector<SharedHandle<RequestGroup> > result;
   createRequestGroupForMetalink(result, requestOption, params[0].s());
   if(!result.empty()) {
     if(posGiven) {
@@ -267,8 +286,8 @@ BDE AddMetalinkXmlRpcMethod::process
       e->_requestGroupMan->addReservedGroup(result);
     }
     BDE gids = BDE::list();
-    for(std::deque<SharedHandle<RequestGroup> >::const_iterator i =
-          result.begin(); i != result.end(); ++i) {
+    for(std::vector<SharedHandle<RequestGroup> >::const_iterator i =
+          result.begin(), eoi = result.end(); i != eoi; ++i) {
       gids << BDE(util::itos((*i)->getGID()));
     }
     return gids;
@@ -278,7 +297,8 @@ BDE AddMetalinkXmlRpcMethod::process
 } 
 #endif // ENABLE_METALINK
 
-BDE RemoveXmlRpcMethod::process(const XmlRpcRequest& req, DownloadEngine* e)
+static BDE removeDownload
+(const XmlRpcRequest& req, DownloadEngine* e, bool forceRemove)
 {
   const BDE& params = req._params;
   assert(params.isList());
@@ -287,7 +307,7 @@ BDE RemoveXmlRpcMethod::process(const XmlRpcRequest& req, DownloadEngine* e)
     throw DL_ABORT_EX(MSG_GID_NOT_PROVIDED);
   }
   
-  int32_t gid = util::parseInt(params[0].s());
+  gid_t gid = util::parseLLInt(params[0].s());
 
   SharedHandle<RequestGroup> group = e->_requestGroupMan->findRequestGroup(gid);
 
@@ -295,19 +315,183 @@ BDE RemoveXmlRpcMethod::process(const XmlRpcRequest& req, DownloadEngine* e)
     group = e->_requestGroupMan->findReservedGroup(gid);
     if(group.isNull()) {
       throw DL_ABORT_EX
-        (StringFormat("Active Download not found for GID#%d", gid).str());
+        (StringFormat("Active Download not found for GID#%s",
+                      util::itos(gid).c_str()).str());
     }
     if(group->isDependencyResolved()) {
       e->_requestGroupMan->removeReservedGroup(gid);
     } else {
       throw DL_ABORT_EX
-        (StringFormat("GID#%d cannot be removed now", gid).str());
+        (StringFormat("GID#%s cannot be removed now",
+                      util::itos(gid).c_str()).str());
     }
   } else {
-    group->setHaltRequested(true, RequestGroup::USER_REQUEST);
+    if(forceRemove) {
+      group->setForceHaltRequested(true, RequestGroup::USER_REQUEST);
+    } else {
+      group->setHaltRequested(true, RequestGroup::USER_REQUEST);
+    }
   }
 
   return createGIDResponse(gid);
+
+}
+
+BDE RemoveXmlRpcMethod::process(const XmlRpcRequest& req, DownloadEngine* e)
+{
+  return removeDownload(req, e, false);
+}
+
+BDE ForceRemoveXmlRpcMethod::process
+(const XmlRpcRequest& req, DownloadEngine* e)
+{
+  return removeDownload(req, e, true);
+}
+
+static bool pauseRequestGroup
+(const SharedHandle<RequestGroup>& group, bool reserved,  bool forcePause)
+{
+  if((reserved && !group->isPauseRequested()) ||
+     (!reserved &&
+      !group->isForceHaltRequested() &&
+      ((forcePause && group->isHaltRequested() && group->isPauseRequested()) ||
+       (!group->isHaltRequested() && !group->isPauseRequested())))) {
+    if(!reserved) {
+      // Call setHaltRequested before setPauseRequested because
+      // setHaltRequested calls setPauseRequested(false) internally.
+      if(forcePause) {
+        group->setForceHaltRequested(true, RequestGroup::NONE);
+      } else {
+        group->setHaltRequested(true, RequestGroup::NONE);
+      }
+    }
+    group->setPauseRequested(true);
+    return true;
+  } else {
+    return false;
+  }
+}
+
+static BDE pauseDownload
+(const XmlRpcRequest& req, DownloadEngine* e, bool forcePause)
+{
+  const BDE& params = req._params;
+  assert(params.isList());
+  if(params.empty() || !params[0].isString()) {
+    throw DL_ABORT_EX(MSG_GID_NOT_PROVIDED);
+  }
+  gid_t gid = util::parseLLInt(params[0].s());
+  bool reserved = false;
+  SharedHandle<RequestGroup> group = e->_requestGroupMan->findRequestGroup(gid);
+  if(group.isNull()) {
+    reserved = true;
+    group = e->_requestGroupMan->findReservedGroup(gid);
+  }
+  if(!group.isNull() && pauseRequestGroup(group, reserved, forcePause)) {
+    return createGIDResponse(gid);
+  } else {
+    throw DL_ABORT_EX
+      (StringFormat("GID#%s cannot be paused now",
+                    util::itos(gid).c_str()).str());
+  }
+}
+
+BDE PauseXmlRpcMethod::process(const XmlRpcRequest& req, DownloadEngine* e)
+{
+  return pauseDownload(req, e, false);
+}
+
+BDE ForcePauseXmlRpcMethod::process(const XmlRpcRequest& req, DownloadEngine* e)
+{
+  return pauseDownload(req, e, true);
+}
+
+template<typename InputIterator>
+static void pauseRequestGroups
+(InputIterator first, InputIterator last, bool reserved, bool forcePause)
+{
+  for(; first != last; ++first) {
+    pauseRequestGroup(*first, reserved, forcePause);
+  }
+}
+
+static BDE pauseAllDownloads
+(const XmlRpcRequest& req, DownloadEngine* e, bool forcePause)
+{
+  const std::deque<SharedHandle<RequestGroup> >& groups =
+    e->_requestGroupMan->getRequestGroups();
+  pauseRequestGroups(groups.begin(), groups.end(), false, forcePause);
+  const std::deque<SharedHandle<RequestGroup> >& reservedGroups =
+    e->_requestGroupMan->getReservedGroups();
+  pauseRequestGroups(reservedGroups.begin(), reservedGroups.end(),
+                     true, forcePause);
+  return BDE_OK;
+}
+
+BDE PauseAllXmlRpcMethod::process(const XmlRpcRequest& req, DownloadEngine* e)
+{
+  return pauseAllDownloads(req, e, false);
+}
+
+BDE ForcePauseAllXmlRpcMethod::process
+(const XmlRpcRequest& req, DownloadEngine* e)
+{
+  return pauseAllDownloads(req, e, true);
+}
+
+BDE UnpauseXmlRpcMethod::process(const XmlRpcRequest& req, DownloadEngine* e)
+{
+  const BDE& params = req._params;
+  assert(params.isList());
+  if(params.empty() || !params[0].isString()) {
+    throw DL_ABORT_EX(MSG_GID_NOT_PROVIDED);
+  }
+  gid_t gid = util::parseLLInt(params[0].s());
+  SharedHandle<RequestGroup> group =e->_requestGroupMan->findReservedGroup(gid);
+  if(group.isNull() || !group->isPauseRequested()) {
+    throw DL_ABORT_EX
+      (StringFormat("GID#%s cannot be unpaused now",
+                    util::itos(gid).c_str()).str());
+  } else {
+    group->setPauseRequested(false);
+    e->_requestGroupMan->requestQueueCheck();    
+  }
+  return createGIDResponse(gid);
+}
+
+BDE UnpauseAllXmlRpcMethod::process(const XmlRpcRequest& req, DownloadEngine* e)
+{
+  const std::deque<SharedHandle<RequestGroup> >& groups =
+    e->_requestGroupMan->getReservedGroups();
+  std::for_each(groups.begin(), groups.end(),
+                std::bind2nd(mem_fun_sh(&RequestGroup::setPauseRequested),
+                             false));
+  e->_requestGroupMan->requestQueueCheck();    
+  return BDE_OK;
+}
+
+static void createUriEntry(BDE& uriList, const SharedHandle<FileEntry>& file)
+{
+  {
+    const std::deque<std::string>& uris = file->getSpentUris();
+    for(std::deque<std::string>::const_iterator i = uris.begin(),
+          eoi = uris.end(); i != eoi; ++i) {
+      BDE entry = BDE::dict();
+      entry[KEY_URI] = *i;
+      entry[KEY_STATUS] = BDE_USED;
+      uriList << entry;
+    }
+  }
+  {
+    const std::deque<std::string>& uris = file->getRemainingUris();
+    for(std::deque<std::string>::const_iterator i = uris.begin(),
+          eoi = uris.end(); i != eoi; ++i) {
+      BDE entry = BDE::dict();
+      entry[KEY_URI] = *i;
+      entry[KEY_STATUS] = BDE_WAITING;
+      uriList << entry;
+    }
+  }
 }
 
 template<typename InputIterator>
@@ -322,14 +506,7 @@ static void createFileEntry(BDE& files, InputIterator first, InputIterator last)
     entry[KEY_LENGTH] = util::uitos((*first)->getLength());
 
     BDE uriList = BDE::list();
-    std::deque<std::string> uris;
-    (*first)->getUris(uris);
-    for(std::deque<std::string>::const_iterator i = uris.begin();
-        i != uris.end(); ++i) {
-      BDE uriEntry = BDE::dict();
-      uriEntry[KEY_URI] = *i;
-      uriList << uriEntry;
-    }
+    createUriEntry(uriList, *first);
     entry[KEY_URIS] = uriList;
     files << entry;
   }
@@ -361,8 +538,8 @@ void gatherProgressCommon
   if(!group->followedBy().empty()) {
     BDE list = BDE::list();
     // The element is GID.
-    for(std::vector<int32_t>::const_iterator i = group->followedBy().begin();
-        i != group->followedBy().end(); ++i) {
+    for(std::vector<gid_t>::const_iterator i = group->followedBy().begin(),
+          eoi = group->followedBy().end(); i != eoi; ++i) {
       list << util::itos(*i);
     }
     entryDict[KEY_FOLLOWED_BY] = list;
@@ -394,11 +571,11 @@ void gatherBitTorrentMetadata(BDE& btDict, const BDE& torrentAttrs)
   // TODO Would it be good to add copy() method in BDE?
   const BDE& announceList = torrentAttrs[bittorrent::ANNOUNCE_LIST];
   BDE destAnnounceList = BDE::list();
-  for(BDE::List::const_iterator l = announceList.listBegin();
-      l != announceList.listEnd(); ++l) {
+  for(BDE::List::const_iterator l = announceList.listBegin(),
+        eoi = announceList.listEnd(); l != eoi; ++l) {
     BDE destAnnounceTier = BDE::list();
-    for(BDE::List::const_iterator t = (*l).listBegin();
-        t != (*l).listEnd(); ++t) {
+    for(BDE::List::const_iterator t = (*l).listBegin(),
+          eoi2 = (*l).listEnd(); t != eoi2; ++t) {
       destAnnounceTier <<  (*t);
     }
     destAnnounceList << destAnnounceTier;
@@ -423,23 +600,28 @@ static void gatherProgressBitTorrent
     SharedHandle<PeerStorage> peerStorage = btObject._peerStorage;
     assert(!peerStorage.isNull());
 
-    std::deque<SharedHandle<Peer> > peers;
+    std::vector<SharedHandle<Peer> > peers;
     peerStorage->getActivePeers(peers);
-    entryDict[KEY_NUM_SEEDERS] = countSeeder(peers.begin(), peers.end());
+    entryDict[KEY_NUM_SEEDERS] =
+      util::uitos(countSeeder(peers.begin(), peers.end()));
   }
 }
 
 static void gatherPeer(BDE& peers, const SharedHandle<PeerStorage>& ps)
 {
-  std::deque<SharedHandle<Peer> > activePeers;
+  std::vector<SharedHandle<Peer> > activePeers;
   ps->getActivePeers(activePeers);
-  for(std::deque<SharedHandle<Peer> >::const_iterator i =
-        activePeers.begin(); i != activePeers.end(); ++i) {
+  for(std::vector<SharedHandle<Peer> >::const_iterator i =
+        activePeers.begin(), eoi = activePeers.end(); i != eoi; ++i) {
     BDE peerEntry = BDE::dict();
-    peerEntry[KEY_PEER_ID] = util::torrentUrlencode((*i)->getPeerId(),
-                                                    PEER_ID_LENGTH);
+    peerEntry[KEY_PEER_ID] = util::torrentPercentEncode((*i)->getPeerId(),
+                                                        PEER_ID_LENGTH);
     peerEntry[KEY_IP] = (*i)->ipaddr;
-    peerEntry[KEY_PORT] = util::uitos((*i)->port);
+    if((*i)->isIncomingPeer()) {
+      peerEntry[KEY_PORT] = std::string("0");
+    } else {
+      peerEntry[KEY_PORT] = util::uitos((*i)->port);
+    }
     peerEntry[KEY_BITFIELD] = util::toHex((*i)->getBitfield(),
                                           (*i)->getBitfieldLength());
     peerEntry[KEY_AM_CHOKING] = (*i)->amChoking()?BDE_TRUE:BDE_FALSE;
@@ -482,8 +664,8 @@ void gatherStoppedDownload
   if(!ds->followedBy.empty()) {
     BDE list = BDE::list();
     // The element is GID.
-    for(std::vector<int32_t>::const_iterator i = ds->followedBy.begin();
-        i != ds->followedBy.end(); ++i) {
+    for(std::vector<gid_t>::const_iterator i = ds->followedBy.begin(),
+          eoi = ds->followedBy.end(); i != eoi; ++i) {
       list << util::itos(*i);
     }
     entryDict[KEY_FOLLOWED_BY] = list;
@@ -496,17 +678,6 @@ void gatherStoppedDownload
   entryDict[KEY_FILES] = files;
 }
 
-static
-SharedHandle<RequestGroup>
-findRequestGroup(const SharedHandle<RequestGroupMan>& rgman, int32_t gid)
-{
-  SharedHandle<RequestGroup> group = rgman->findRequestGroup(gid);
-  if(group.isNull()) {
-    group = rgman->findReservedGroup(gid);
-  }
-  return group;
-}
-
 BDE GetFilesXmlRpcMethod::process
 (const XmlRpcRequest& req, DownloadEngine* e)
 {
@@ -517,7 +688,7 @@ BDE GetFilesXmlRpcMethod::process
     throw DL_ABORT_EX(MSG_GID_NOT_PROVIDED);
   }
   
-  int32_t gid = util::parseInt(params[0].s());
+  gid_t gid = util::parseLLInt(params[0].s());
 
   BDE files = BDE::list();
   SharedHandle<RequestGroup> group = findRequestGroup(e->_requestGroupMan, gid);
@@ -526,7 +697,8 @@ BDE GetFilesXmlRpcMethod::process
       e->_requestGroupMan->findDownloadResult(gid);
     if(dr.isNull()) {
       throw DL_ABORT_EX
-        (StringFormat("No file data is available for GID#%d", gid).str());
+        (StringFormat("No file data is available for GID#%s",
+                      util::itos(gid).c_str()).str());
     } else {
       createFileEntry(files, dr->fileEntries.begin(), dr->fileEntries.end());
     }
@@ -548,24 +720,18 @@ BDE GetUrisXmlRpcMethod::process
     throw DL_ABORT_EX(MSG_GID_NOT_PROVIDED);
   }
   
-  int32_t gid = util::parseInt(params[0].s());
+  gid_t gid = util::parseLLInt(params[0].s());
 
   SharedHandle<RequestGroup> group = findRequestGroup(e->_requestGroupMan, gid);
   if(group.isNull()) {
     throw DL_ABORT_EX
-      (StringFormat("No URI data is available for GID#%d", gid).str());
+      (StringFormat("No URI data is available for GID#%s",
+                    util::itos(gid).c_str()).str());
   }
   BDE uriList = BDE::list();
   // TODO Current implementation just returns first FileEntry's URIs.
   if(!group->getDownloadContext()->getFileEntries().empty()) {
-    std::deque<std::string> uris;
-    group->getDownloadContext()->getFirstFileEntry()->getUris(uris);
-    for(std::deque<std::string>::const_iterator i = uris.begin();
-        i != uris.end(); ++i) {
-      BDE entry = BDE::dict();
-      entry[KEY_URI] = *i;
-      uriList << entry;
-    }
+    createUriEntry(uriList, group->getDownloadContext()->getFirstFileEntry());
   }
   return uriList;
 }
@@ -581,12 +747,13 @@ BDE GetPeersXmlRpcMethod::process
     throw DL_ABORT_EX(MSG_GID_NOT_PROVIDED);
   }
   
-  int32_t gid = util::parseInt(params[0].s());
+  gid_t gid = util::parseLLInt(params[0].s());
 
   SharedHandle<RequestGroup> group = findRequestGroup(e->_requestGroupMan, gid);
   if(group.isNull()) {
     throw DL_ABORT_EX
-      (StringFormat("No peer data is available for GID#%d", gid).str());
+      (StringFormat("No peer data is available for GID#%s",
+                    util::itos(gid).c_str()).str());
   }
   BDE peers = BDE::list();
   BtObject btObject = e->getBtRegistry()->get(group->getGID());
@@ -608,7 +775,7 @@ BDE TellStatusXmlRpcMethod::process
     throw DL_ABORT_EX(MSG_GID_NOT_PROVIDED);
   }
   
-  int32_t gid = util::parseInt(params[0].s());
+  gid_t gid = util::parseLLInt(params[0].s());
 
   SharedHandle<RequestGroup> group = e->_requestGroupMan->findRequestGroup(gid);
 
@@ -620,11 +787,16 @@ BDE TellStatusXmlRpcMethod::process
         e->_requestGroupMan->findDownloadResult(gid);
       if(ds.isNull()) {
         throw DL_ABORT_EX
-          (StringFormat("No such download for GID#%d", gid).str());
+          (StringFormat("No such download for GID#%s",
+                        util::itos(gid).c_str()).str());
       }
       gatherStoppedDownload(entryDict, ds);
     } else {
-      entryDict[KEY_STATUS] = BDE_WAITING;
+      if(group->isPauseRequested()) {
+        entryDict[KEY_STATUS] = BDE_PAUSED;
+      } else {
+        entryDict[KEY_STATUS] = BDE_WAITING;
+      }
       gatherProgress(entryDict, group, e);
     }
   } else {
@@ -641,7 +813,7 @@ BDE TellActiveXmlRpcMethod::process
   const std::deque<SharedHandle<RequestGroup> >& groups =
     e->_requestGroupMan->getRequestGroups();
   for(std::deque<SharedHandle<RequestGroup> >::const_iterator i =
-        groups.begin(); i != groups.end(); ++i) {
+        groups.begin(), eoi = groups.end(); i != eoi; ++i) {
     BDE entryDict = BDE::dict();
     entryDict[KEY_STATUS] = BDE_ACTIVE;
     gatherProgress(entryDict, *i, e);
@@ -660,7 +832,11 @@ void TellWaitingXmlRpcMethod::createEntry
 (BDE& entryDict, const SharedHandle<RequestGroup>& item,
  DownloadEngine* e) const
 {
-  entryDict[KEY_STATUS] = BDE_WAITING;
+  if(item->isPauseRequested()) {
+    entryDict[KEY_STATUS] = BDE_PAUSED;
+  } else {
+    entryDict[KEY_STATUS] = BDE_WAITING;
+  }
   gatherProgress(entryDict, item, e);
 }
 
@@ -692,12 +868,13 @@ BDE ChangeOptionXmlRpcMethod::process
   if(params.empty() || !params[0].isString()) {
     throw DL_ABORT_EX(MSG_GID_NOT_PROVIDED);
   }  
-  int32_t gid = util::parseInt(params[0].s());
+  gid_t gid = util::parseLLInt(params[0].s());
 
   SharedHandle<RequestGroup> group = findRequestGroup(e->_requestGroupMan, gid);
   if(group.isNull()) {
     throw DL_ABORT_EX
-      (StringFormat("Cannot change option for GID#%d", gid).str());
+      (StringFormat("Cannot change option for GID#%s",
+                    util::itos(gid).c_str()).str());
   }
   SharedHandle<Option> option(new Option());
   if(params.size() > 1 && params[1].isDict()) {
@@ -745,6 +922,7 @@ BDE ChangeGlobalOptionXmlRpcMethod::process
   if(option->defined(PREF_MAX_CONCURRENT_DOWNLOADS)) {
     e->_requestGroupMan->setMaxSimultaneousDownloads
       (option->getAsInt(PREF_MAX_CONCURRENT_DOWNLOADS));
+    e->_requestGroupMan->requestQueueCheck();
   }
   return BDE_OK;
 }
@@ -756,7 +934,8 @@ BDE GetVersionXmlRpcMethod::process
   result[KEY_VERSION] = std::string(PACKAGE_VERSION);
   BDE featureList = BDE::list();
   const FeatureMap& features = FeatureConfig::getInstance()->getFeatures();
-  for(FeatureMap::const_iterator i = features.begin(); i != features.end();++i){
+  for(FeatureMap::const_iterator i = features.begin(), eoi = features.end();
+      i != eoi;++i){
     if((*i).second) {
       featureList << (*i).first;
     }
@@ -785,12 +964,13 @@ BDE GetOptionXmlRpcMethod::process
   if(params.empty() || !params[0].isString()) {
     throw DL_ABORT_EX(MSG_GID_NOT_PROVIDED);
   }  
-  int32_t gid = util::parseInt(params[0].s());
+  gid_t gid = util::parseLLInt(params[0].s());
 
   SharedHandle<RequestGroup> group = findRequestGroup(e->_requestGroupMan, gid);
   if(group.isNull()) {
     throw DL_ABORT_EX
-      (StringFormat("Cannot get option for GID#%d", gid).str());
+      (StringFormat("Cannot get option for GID#%s",
+                    util::itos(gid).c_str()).str());
   }
   BDE result = BDE::dict();
   SharedHandle<Option> option = group->getOption();
@@ -802,8 +982,8 @@ BDE GetGlobalOptionXmlRpcMethod::process
 (const XmlRpcRequest& req, DownloadEngine* e)
 {
   BDE result = BDE::dict();
-  for(std::map<std::string, std::string>::const_iterator i = e->option->begin();
-      i != e->option->end(); ++i) {
+  for(std::map<std::string, std::string>::const_iterator i = e->option->begin(),
+        eoi = e->option->end(); i != eoi; ++i) {
     SharedHandle<OptionHandler> h = _optionParser->findByName((*i).first);
     if(!h.isNull() && !h->isHidden()) {
       result[(*i).first] = (*i).second;
@@ -822,7 +1002,7 @@ BDE ChangePositionXmlRpcMethod::process
      !params[0].isString() || !params[1].isInteger() || !params[2].isString()) {
     throw DL_ABORT_EX("Illegal argument.");
   }
-  int32_t gid = util::parseInt(params[0].s());
+  gid_t gid = util::parseLLInt(params[0].s());
   int pos = params[1].i();
   const std::string& howStr = params[2].s();
   RequestGroupMan::HOW how;
@@ -849,6 +1029,139 @@ BDE GetSessionInfoXmlRpcMethod::process
   return result;
 }
 
+BDE GetServersXmlRpcMethod::process
+(const XmlRpcRequest& req, DownloadEngine* e)
+{
+  const BDE& params = req._params;
+  assert(params.isList());
+
+  if(params.empty() || !params[0].isString()) {
+    throw DL_ABORT_EX("Bad request");
+  }
+  gid_t gid = util::parseLLInt(params[0].s());
+  SharedHandle<RequestGroup> group = e->_requestGroupMan->findRequestGroup(gid);
+  if(group.isNull()) {
+    throw DL_ABORT_EX(StringFormat("No active download for GID#%s",
+                                   util::itos(gid).c_str()).str());
+  }
+  SharedHandle<DownloadContext> dctx = group->getDownloadContext();
+  const std::vector<SharedHandle<FileEntry> >& files = dctx->getFileEntries();
+  BDE result = BDE::list();
+  size_t index = 1;
+  for(std::vector<SharedHandle<FileEntry> >::const_iterator fi = files.begin(),
+        eoi = files.end(); fi != eoi; ++fi, ++index) {
+    BDE fileEntry = BDE::dict();
+    fileEntry[KEY_INDEX] = util::uitos(index);
+    BDE servers = BDE::list();
+    const std::deque<SharedHandle<Request> >& requests =
+      (*fi)->getInFlightRequests();
+    for(std::deque<SharedHandle<Request> >::const_iterator ri =requests.begin(),
+          eoi = requests.end(); ri != eoi; ++ri) {
+      SharedHandle<PeerStat> ps = (*ri)->getPeerStat();
+      if(!ps.isNull()) {
+        BDE serverEntry = BDE::dict();
+        serverEntry[KEY_URI] = (*ri)->getUri();
+        serverEntry[KEY_CURRENT_URI] = (*ri)->getCurrentUri();
+        serverEntry[KEY_DOWNLOAD_SPEED] =
+          util::uitos(ps->calculateDownloadSpeed());
+        servers << serverEntry;
+      }
+    }
+    fileEntry[KEY_SERVERS] = servers;
+    result << fileEntry;
+  }
+  return result;
+}
+
+BDE ChangeUriXmlRpcMethod::process
+(const XmlRpcRequest& req, DownloadEngine* e)
+{
+  const BDE& params = req._params;
+  assert(params.isList());
+
+  if(params.size() < 4 ||
+     !params[0].isString() || !params[1].isInteger() ||
+     !params[2].isList() || !params[3].isList() ||
+     params[1].i() <= 0) {
+    throw DL_ABORT_EX("Bad request");
+  }
+  size_t pos = 0;
+  bool posGiven = false;
+  getPosParam(params, 4, posGiven, pos);
+
+  gid_t gid = util::parseLLInt(params[0].s());
+  size_t index = params[1].i()-1;
+  const BDE& deluris = params[2];
+  const BDE& adduris = params[3];
+  SharedHandle<RequestGroup> group = findRequestGroup(e->_requestGroupMan, gid);
+  if(group.isNull()) {
+    throw DL_ABORT_EX
+      (StringFormat("Cannot remove URIs from GID#%s",
+                    util::itos(gid).c_str()).str());
+  }
+  SharedHandle<DownloadContext> dctx = group->getDownloadContext();
+  const std::vector<SharedHandle<FileEntry> >& files = dctx->getFileEntries();
+  if(files.size() <= index) {
+    throw DL_ABORT_EX(StringFormat("fileIndex is out of range").str());
+  }
+  SharedHandle<FileEntry> s = files[index];
+  size_t delcount = 0;
+  for(BDE::List::const_iterator i = deluris.listBegin(),
+        eoi = deluris.listEnd(); i != eoi; ++i) {
+    if(s->removeUri((*i).s())) {
+      ++delcount;
+    }
+  }
+  size_t addcount = 0;
+  if(posGiven) {
+    for(BDE::List::const_iterator i = adduris.listBegin(),
+          eoi = adduris.listEnd(); i != eoi; ++i) {
+      if(s->insertUri((*i).s(), pos)) {
+        ++addcount;
+        ++pos;
+      }
+    }
+  } else {
+    for(BDE::List::const_iterator i = adduris.listBegin(),
+          eoi = adduris.listEnd(); i != eoi; ++i) {
+      if(s->addUri((*i).s())) {
+        ++addcount;
+      }
+    }
+  }
+  if(addcount && !group->getPieceStorage().isNull()) {
+    std::vector<Command*> commands;
+    group->createNextCommand(commands, e);
+    e->addCommand(commands);
+    group->getSegmentMan()->recognizeSegmentFor(s);
+  }
+  BDE res = BDE::list();
+  res << delcount;
+  res << addcount;
+  return res;
+}
+
+static BDE goingShutdown
+(const XmlRpcRequest& req, DownloadEngine* e, bool forceHalt)
+{
+  // Schedule shutdown after 3seconds to give time to client to
+  // receive XML-RPC response.
+  e->addRoutineCommand(new TimedHaltCommand(e->newCUID(), e, 3, forceHalt));
+  LogFactory::getInstance()->info("Scheduled shutdown in 3 seconds.");
+  return BDE_OK;
+}
+
+BDE ShutdownXmlRpcMethod::process(const XmlRpcRequest& req, DownloadEngine* e)
+{
+  return goingShutdown(req, e, false);
+}
+
+BDE ForceShutdownXmlRpcMethod::process
+(const XmlRpcRequest& req, DownloadEngine* e)
+{
+  return goingShutdown(req, e, true);
+}
+
 BDE SystemMulticallXmlRpcMethod::process
 (const XmlRpcRequest& req, DownloadEngine* e)
 {
@@ -860,8 +1173,8 @@ BDE SystemMulticallXmlRpcMethod::process
   }
   const BDE& methodSpecs = params[0];
   BDE list = BDE::list();
-  for(BDE::List::const_iterator i = methodSpecs.listBegin();
-      i != methodSpecs.listEnd(); ++i) {
+  for(BDE::List::const_iterator i = methodSpecs.listBegin(),
+        eoi = methodSpecs.listEnd(); i != eoi; ++i) {
     if(!(*i).isDict()) {
       list << createErrorResponse
         (DL_ABORT_EX("system.multicall expected struct."));
